@@ -194,6 +194,13 @@ static void powered_changed(struct connman_device *device)
 					DBUS_TYPE_BOOLEAN, &device->powered);
 }
 
+static void blocked_changed(struct connman_device *device)
+{
+	connman_dbus_property_changed_basic(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "Blocked",
+					DBUS_TYPE_BOOLEAN, &device->blocked);
+}
+
 int __connman_device_enable(struct connman_device *device)
 {
 	int err;
@@ -333,6 +340,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_basic(&dict, "Powered",
 					DBUS_TYPE_BOOLEAN, &device->powered);
 
+	connman_dbus_dict_append_basic(&dict, "Blocked",
+					DBUS_TYPE_BOOLEAN, &device->blocked);
+
 	if (device->driver && device->driver->scan)
 		connman_dbus_dict_append_basic(&dict, "Scanning",
 					DBUS_TYPE_BOOLEAN, &device->scanning);
@@ -585,9 +595,13 @@ static void probe_driver(struct connman_element *element, gpointer user_data)
 
 static void remove_device(struct connman_device *device)
 {
+	int err;
+
 	DBG("device %p", device);
 
-	__connman_device_disable(device);
+	err = __connman_device_disable(device);
+	if (err < 0 && err == -EINPROGRESS)
+		__connman_technology_disable_device(device);
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
@@ -989,8 +1003,10 @@ int connman_device_set_powered(struct connman_device *device,
 		device->pending = NULL;
 	}
 
-	if (device->powered == powered)
+	if (device->powered == powered) {
+		device->powered_pending = powered;
 		return -EALREADY;
+	}
 
 	if (powered == TRUE)
 		__connman_device_enable(device);
@@ -1005,6 +1021,11 @@ int connman_device_set_powered(struct connman_device *device,
 	else
 		__connman_technology_disable_device(device);
 
+	if (device->offlinemode == TRUE && powered == TRUE) {
+		powered_changed(device);
+		return connman_device_set_powered(device, FALSE);
+	}
+
 	if (device->registered == FALSE)
 		return 0;
 
@@ -1030,6 +1051,8 @@ int __connman_device_set_blocked(struct connman_device *device,
 
 	device->blocked = blocked;
 
+	blocked_changed(device);
+
 	if (device->offlinemode == TRUE)
 		return 0;
 
@@ -1043,6 +1066,11 @@ int __connman_device_set_blocked(struct connman_device *device,
 	return set_powered(device, powered);
 }
 
+connman_bool_t __connman_device_get_blocked(struct connman_device *device)
+{
+	return device->blocked;
+}
+
 int __connman_device_scan(struct connman_device *device)
 {
 	if (!device->driver || !device->driver->scan)
@@ -1068,9 +1096,12 @@ int __connman_device_enable_persistent(struct connman_device *device)
 
 	err = __connman_device_enable(device);
 	if (err == 0 || err == -EINPROGRESS) {
-		if (__connman_profile_get_offlinemode() == TRUE)
+		device->offlinemode = FALSE;
+		if (__connman_profile_get_offlinemode() == TRUE) {
 			__connman_profile_set_offlinemode(FALSE, FALSE);
 
+			__connman_profile_save_default();
+		}
 	}
 
 	return err;
